export interface BinpatOption {
    /** The global endianness. Defalut value is `big`. */
    endian: 'big' | 'little';
}
export type BinpatPattern = any;
export interface BinpatContext {
    endian: BinpatOption['endian'];
    parent?: BinpatContext;
    offset: number;
    data: any;
}
declare const BINPAT_TYPE: unique symbol;
declare const BINPAT_ARRAY: unique symbol;
export default class Binpat {
    #private;
    endian: BinpatOption['endian'];
    constructor(pattern: BinpatPattern, option?: BinpatOption);
    exec(buffer: ArrayBuffer): any;
}
type TypedArrayConstructor = Uint8ArrayConstructor | Uint16ArrayConstructor | Uint32ArrayConstructor | BigUint64ArrayConstructor | Int8ArrayConstructor | Int16ArrayConstructor | Int32ArrayConstructor | BigInt64ArrayConstructor | Float16ArrayConstructor | Float32ArrayConstructor | Float64ArrayConstructor;
type BinpatTypeLe<T> = (
/** Indicates whether the data is stored in little- or big-endian format. If `undefined`, the global endianness is used. */
littleEndian?: boolean) => {
    (dv: DataView, ctx: BinpatContext): T;
    [BINPAT_TYPE]: string;
    [BINPAT_ARRAY]: TypedArrayConstructor;
};
/**
 * Reads 1 byte and interprets it as an 8-bit unsigned integer.
 */
export declare const u8: () => {
    (dv: DataView, ctx: BinpatContext): number;
    [BINPAT_TYPE]: string;
    [BINPAT_ARRAY]: TypedArrayConstructor;
};
/**
 * Reads 2 bytes and interprets them as a 16-bit unsigned integer.
 */
export declare const u16: BinpatTypeLe<number>;
/**
 * Reads 4 bytes and interprets them as a 32-bit unsigned integer.
 */
export declare const u32: BinpatTypeLe<number>;
/**
 * Reads 8 bytes and interprets them as a 64-bit unsigned integer.
 */
export declare const u64: BinpatTypeLe<bigint>;
/**
 * Reads 1 byte and interprets it as an 8-bit signed integer.
 */
export declare const i8: () => {
    (dv: DataView, ctx: BinpatContext): number;
    [BINPAT_TYPE]: string;
    [BINPAT_ARRAY]: TypedArrayConstructor;
};
/**
 * Reads 2 bytes and interprets them as a 16-bit signed integer.
 */
export declare const i16: BinpatTypeLe<number>;
/**
 * Reads 4 bytes and interprets them as a 32-bit signed integer.
 */
export declare const i32: BinpatTypeLe<number>;
/**
 * Reads 8 bytes and interprets them as a 64-bit signed integer.
 */
export declare const i64: BinpatTypeLe<bigint>;
/**
 * Reads 2 bytes and interprets them as a 16-bit floating point number.
 */
export declare const f16: BinpatTypeLe<number>;
/**
 * Reads 4 bytes and interprets them as a 32-bit floating point number.
 */
export declare const f32: BinpatTypeLe<number>;
/**
 * Reads 8 bytes and interprets them as a 64-bit floating point number.
 */
export declare const f64: BinpatTypeLe<bigint>;
/**
 * Reads 1 byte and and interprets it as a boolean value.
 */
export declare function bool(): {
    (dv: DataView, ctx: BinpatContext): boolean;
    [BINPAT_TYPE]: string;
};
/**
 * Reads the given number of bytes and decodes to a string.
 * @param size number of bytes
 * @param encoding same as [TextDecoder.encoding](https://developer.mozilla.org/en-US/docs/Web/API/TextDecoder/encoding)
 *
 * @example Usage
 * ```js
 * import Binpat, { string } from 'binpat';
 *
 * const binpat = new Binpat({ name: string(6) });
 * const { buffer } = new Uint8Array([98, 105, 110, 112, 97, 116]);
 * console.log(binpat.exec(buffer));
 * // { name: 'binpat' }
 * ```
 */
export declare function string(size: number | ((ctx: BinpatContext) => number), encoding?: TextDecoderCommon['encoding']): {
    (dv: DataView, ctx: BinpatContext): string;
    [BINPAT_TYPE]: string;
};
/**
 * Reads multiple values of the same pattern.
 * @param pattern
 * @param size
 *
 * @example Usage
 * ```
 * import Binpat, { array, u8 } from 'binpat';
 *
 * const binpat = new Binpat({
 *   // It will return Uint16Array
 *   foo: array(u16(), 4),
 *   // pattern can be object
 *   bar: array({ x: u8(), y: u8() }, 4),
 *   count: u32(),
 *   // get size from context
 *   baz: array(u8(), (ctx) => ctx.data.count),
 * });
 * ```
 */
export declare function array(pattern: BinpatPattern, size: number | ((ctx: BinpatContext) => number)): {
    (dv: DataView, ctx: BinpatContext): any;
    [BINPAT_TYPE]: string;
};
/**
 * It works like `condition(ctx) ? truthy : falsy`
 * @param condition
 * @param truthy
 * @param falsy
 *
 * @example Usage
 * ```js
 * import Binpat, { ternary, bool, u16, u8 } from 'binpat';
 *
 * const binpat = new Binpat({
 *   flag: bool(),
 *   value: ternary((ctx) => ctx.data.flag, [u8(), u8(), u8(), u8()], [u16(), u16]),
 * });
 * console.log(binpat.exec(buffer));
 * // { flag: true, value: [0, 0, 0, 0] }
 * // or
 * // { flag: false, value: [0, 0] }
 * ```
 */
export declare function ternary(condition: (ctx: BinpatContext) => boolean, truthy: BinpatPattern, falsy?: BinpatPattern): {
    (dv: DataView, ctx: BinpatContext): any;
    [BINPAT_TYPE]: string;
};
/**
 * Convert the result value with custom function.
 * @param pattern
 * @param fn
 *
 * @example Usage
 * ```js
 * import Binpat, { convert } from 'binpat';
 *
 * const binpat = new Binpat({
 *   type: convert(u16(), (value) => ['', 'ico', 'cur'][value] || 'unknown'),
 * });
 * console.log(binpat.exec(buffer));
 * // { type: 'ico' }
 * // or
 * // { type: 'cur' }
 * // or
 * // { type: 'unknown' }
 * ```
 */
export declare function convert(pattern: BinpatPattern, fn: (value: any, ctx?: BinpatContext) => any): {
    (dv: DataView, ctx: BinpatContext): any;
    [BINPAT_TYPE]: string;
};
/**
 * Move current offset to the given offset.
 * @param offset
 *
 * @example Usage
 * ```js
 * import Binpat, { seek, omit, u8 } from 'binpat';
 *
 * const binpat = new Binpat({
 *   foo: u8(),
 *   [omit('padding')]: seek((ctx) => ctx.offset + 4),
 *   bar: u8(),
 * });
 * const { buffer } = new Uint8Array([1, 0, 0, 0, 0, 2]);
 * console.log(binpat.exec(buffer));
 * // { foo: 1, bar: 2 }
 * ```
 */
export declare function seek(offset: number | ((ctx: BinpatContext) => number)): {
    (dv: DataView, ctx: BinpatContext): void;
    [BINPAT_TYPE]: string;
};
/**
 * Reads pattern from the given offset, and doesn't move current offset.
 * @param offset
 * @param pattern
 *
 * @example Usage
 * ```js
 * import Binpat, { peek, u8 } from 'binpat';
 *
 * const binpat = new Binpat({
 *   size: u32(),
 *   address: u32(),
 *   data: peek(
 *     (ctx) => ctx.data.address,
 *     array(u8(), (ctx) => ctx.data.size),
 *   ),
 * });
 * ```
 */
export declare function peek(offset: number | ((ctx: BinpatContext) => number), pattern: any): {
    (dv: DataView, ctx: BinpatContext): any;
    [BINPAT_TYPE]: string;
};
/**
 * Move forward with the given offset. `skip(x)` is same as `seek((ctx) => ctx.offset + x)`
 * @param offset
 *
 * @example Usage
 * ```js
 * import Binpat, { skip, omit, u8 } from 'binpat';
 *
 * const binpat = new Binpat({
 *   foo: u8(),
 *   [omit('padding')]: skip(4),
 *   bar: u8(),
 * });
 * const { buffer } = new Uint8Array([1, 0, 0, 0, 0, 2]);
 * console.log(binpat.exec(buffer));
 * // { foo: 1, bar: 2 }
 * ```
 */
export declare function skip(offset: number): {
    (dv: DataView, ctx: BinpatContext): void;
    [BINPAT_TYPE]: string;
};
/**
 * Omit the key-value in result.
 *
 * @example Usage
 * ```js
 * import Binpat, { omit, u16 } from 'binpat';
 *
 * const binpat = new Binpat({
 *   [omit('reserved')]: u16(),
 *   type: u16(),
 *   count: u16(),
 * });
 * console.log(binpat.exec(buffer));
 * // { type: 1, count: 1 }
 * ```
 */
export declare function omit(name?: string): string;
/**
 * It works like spread syntax `...`, and usually be used with `ternary()`.
 *
 * @example Usage
 * ```js
 * import Binpat, { spread, ternary, bool, u8 } from 'binpat';
 *
 * const binpat = new Binpat({
 *   flag: bool(),
 *   [spread()]: ternary(
 *     (ctx) => ctx.data.flag,
 *     { truthy: u8() },
 *     { falsy: u8() },
 *   ),
 * });
 * console.log(binpat.exec(buffer));
 * // { flag: true, truthy: 0 }
 * // or
 * // { flag: false, falsy: 0 }
 * ```
 */
export declare function spread(): string;
export {};
